
; Bend Development Tool
;   Uses the k-factor method with the
;   specified k-factor and metal thickness
;   to calculate the developed length of
;   a part by selecting the lines and
;   arcs from a cross-section of the part.

;Declare Global Variables
;*BA       ;Bend angle (in degrees)
;*IBR      ;Inside bend radius
;*KF       ;k-Factor
;*TH       ;Metal thickness

;Define the application
(defun C:kesim_olcusu
  (/

  ;Declare Local Variables
  devlen   ;Total developed length
  entl     ;Entity dxf list
  inp      ;Result of (getkword)
  olderr   ;Original error handler
  sel      ;Result of (entget)

  ;Declare Local Functions
  adddev   ;Add and display developed length
  dxf      ;Extract dxf code
  errexit  ;Error handler
  numchk   ;Validate number
  numinp   ;Input valid number
  )

  ;Define Local Functions

  ;============================================
  ;= (adddev <length_to_add>)                 =
  ;=   adds <length_to_add> to 'devlen' and   =
  ;=   prints the new total.                  =
  ;============================================
  (defun adddev (addlen)
    (princ
      (strcat
        "\nTotal developed length is "
        (rtos
          (setq devlen (+ devlen addlen))
          (getvar "LUNITS")
          (+ 2 (getvar "LUPREC"))))))

  ;============================================
  ;= (dxf <dxf_code> <entity_list>)           =
  ;=   extracts the value of the <dxf_code>   =
  ;=   group from <entity_list>.              =
  ;============================================
  (defun dxf (grp elist)
    (cdr (assoc grp elist)))

  ;============================================
  ;= (errexit <message>)                      =
  ;=   processes error <message>.             =
  ;============================================
  (defun errexit (s)
    (setq *error* olderr)
    (if (and *error* (listp *error*))
      (*error* s)
      (if
        (not
          (member s
            '("console break"
              "Function cancelled"
              "quit / exit abort")))
	      (princ (strcat "\nError:  " s))))
    (princ))
  ;============================================
  ;= (numchk <number>)                        =
  ;=   returns T if <number> is a positive    =
  ;=   number or zero.                        =
  ;============================================
  (defun numchk (num)
    (and (numberp num) (>= num 0)))

  ;============================================
  ;= (numinp <prompt> <default>)              =
  ;=   gets input and validates it with       =
  ;=   (numchk) before returning it.          =
  ;============================================
  (defun numinp (prompt default / inp)
    (while
      (and
        (setq inp
          (getdist
            (strcat
              "\n"
              prompt
              "<"
              (rtos
                default
                (getvar "LUNITS")
                (+ 2 (getvar "LUPREC")))
              ">: ")))
        (not (numchk inp))))
    (cond (inp) (default)))

  ;set up our error handler
  (setq olderr *error* *error* errexit)

  ;check global variables for valid values
  ;and set to defaults if necessary
  (if (not (numchk *TH))
    (setq *TH 0.0598))

  ;get input and store in global variables
  (setq *TH
        (numinp "Material Thickness" *TH)
        *KF ;set k-Factor to 40%
        0.40)    ;[or, could use (numinp)]

  ;reset the running blank length total
  (setq devlen 0)
(vl-load-com)

(defun DP (side / *error* doc svnames svvals styht dpss n pl cw inc pt1 pt3 pt2 ang1 ang2 dtxt pt4)

  (defun *error* (errmsg)
    (if (not (wcmatch errmsg "Function cancelled,quit / exit abort,console break"))
      (princ (strcat "\nError: " errmsg))
    ); if
    (mapcar 'setvar svnames svvals); reset System Variables
    (vla-endundomark doc)
    (princ)
  ); defun -- *error*

  (vla-startundomark (setq doc (vla-get-activedocument (vlax-get-acad-object))))
  (setq ; System Variable saving/resetting without separate variables for each:
    svnames '(osmode cmdecho blipmode clayer)
    svvals (mapcar 'getvar svnames)
  ); setq
  (mapcar 'setvar svnames '(0 0)); turn off Osnap, command echoing
  (command "_.layer" "_make" "A-ANNO-DIMS" "_color" 2 "" "");; <---EDIT Layer name/color as desired
  (setq styht (cdr (assoc 40 (tblsearch "style" (getvar 'dimtxsty))))); height of text style in current dimension style
  (if (= styht 0.0) (setq styht (* (getvar 'dimtxt) (getvar 'dimscale)))); if above is non-fixed-height

  (prompt (strcat "\nTo Dimension Polyline(s) on the " side "side,"))
  (if (setq dpss (ssget "_:L" '((0 . "*POLYLINE"))))
    (progn ; then [can omit this if you never use blips; also two other lines below]
      (setvar 'blipmode 0); [omit if you never use blips and omitted line above]
      (repeat (setq n (sslength dpss)); step through selection set
        (setq pl (ssname dpss (setq n (1- n))))
        (if (= (logand (cdr (assoc 70 (entget pl))) 88) 0)
          ;; not 3D or mesh [88 = 8 (3D) + 16 (polygon mesh) + 64 (polyface mesh)]
          (progn ; then
            (setq pl (vlax-ename->vla-object pl))
            (vla-offset pl styht); temporary
            (setq cw (< (vla-get-area (vlax-ename->vla-object (entlast))) (vla-get-area pl)))
              ;; clockwise for closed or clearly inside/outside open; may not give
              ;; desired result for open without obvious inside/outside
            (entdel (entlast))
            (repeat (setq inc (fix (vlax-curve-getEndParam pl))); segments
              (setq
                pt1 (vlax-curve-getPointAtParam pl inc)
                pt3 (vlax-curve-getPointAtParam pl (1- inc))
              ); setq
              (if (not (equal pt1 pt3 1e-8)); not coincident vertices
                (progn ; then -- proceed to dimension segment
                  (setq
                    pt2 (vlax-curve-getPointAtParam pl (- inc 0.5)); segment midpoint
                    ang1 (angle pt1 pt2)
                    ang2 (angle pt2 pt3)
                  ); setq
                  (if
                    (or ; line segment?
                      (equal ang1 ang2 1e-8); any non-0 direction or both reading as 0 or 2 pi
                      (equal (abs (- ang2 ang1)) (* pi 2) 1e-8); 0-degree with one reading as 2 pi +/-
                    ); or
                    (command "_.dimaligned" pt1 pt3); then [leaves at dimension line location prompt]
                    (progn ; else [arc segment]
                      (setq dtxt
                        (rtos
                          (abs ; length along arc segment
                            (- (vlax-curve-getDistAtParam pl inc) (vlax-curve-getDistAtParam pl (1- inc)))
                          ); abs
                          (getvar 'DIMLUNITS) 2 ;; [include mode/precision here if current dimension style's settings not desired]
                        ); rtos
                      ); setq
                      (if (wcmatch dtxt "*/*"); includes fraction?
                        ;; can omit this entire (if) function if you don't use stacked fractions
                        (setq dtxt ; stack it
                          (strcat
                            "\\A1;"
                            (vl-string-subst ";}\"" "\""
;                              (vl-string-subst "#" "/"
                                ;; remove ; from beginning of above line and its closing parenthesis line
                                ;; below to make diagonal stack [makes horizontal-line stack without]
                                (vl-string-subst "{\\H0.875x;\\S" " " dtxt)
                                  ;; change 0.875 ratio to agree with Dimension Style's setting
;                              ); -subst
                            ); -subst
                          ); strcat & dtxt
                        ); setq
                      ); if
                      (command
                        "_.dimangular" ""
                        (inters ; arc center
                          (setq pt4 (mapcar '/ (mapcar '+ pt1 pt2) '(2 2 2)))
                          (polar pt4 (+ (angle pt1 pt2) (/ pi 2)) 1)
                          (setq pt4 (mapcar '/ (mapcar '+ pt2 pt3) '(2 2 2)))
                          (polar pt4 (+ (angle pt2 pt3) (/ pi 2)) 1)
                          nil
                        ); inters
                        pt1 pt3
                        "_text" dtxt
                      ); command [leaves at dimension line location prompt]
                    ); progn
                  ); if
                  (command ; complete Dimension: dimension line location
                    (polar
                      pt2
                      (apply ; angle
                        (if (or (and cw (= side "in")) (and (not cw) (= side "out"))) '- '+)
                        (list
                          (angle '(0 0 0) (vlax-curve-getFirstDeriv pl (- inc 0.5)))
                          (/ pi 2)
                        ); list
                      ); apply
                      (* styht 1.5); distance
                        ;; [If you don't use stacked fractions, consider using styht without multiplier]
                    ); polar
                  ); command
                ); progn
              ); if [not coincident]
              (setq inc (1- inc))
            ); repeat [segments]
          ); progn -- then [LW/2D]
        ); if
      ); repeat [Polylines]
    ); progn -- then [valid selection]; [omit if you never use blips and omitted 2 lines above]
    (prompt "\nNo Polyline(s) on unlocked Layer(s) selected."); else
  ); if

  (mapcar 'setvar svnames svvals); reset System Variables
  (vla-endundomark doc)
  (princ)
); defun -- DP

  ;loop until the user is finished selecting
  (while
    (or
      (setq sel (entsel "\nPick a segment: "))
      (/= (getvar "ERRNO") 52))
    (cond
      ( (not sel)
        (princ "  <Nothing selected>"))
      ( (= "PLINE"
          (dxf 0
            (setq entl (entget (car sel)))))
        (adddev
          (distance
            (dxf 10 entl)
            (dxf 11 entl))))
      ( (= "ARC" (dxf 0 entl))
        (initget "Inside Outside")
        (setq inp
          (getkword
            (strcat
              "\nIs this arc ("
              "<Inside>/Outside"
              ")?  ")))
        (setq *IBR
          (if (= inp "Outside")
            (- (dxf 40 entl) *TH)
            (dxf 40 entl)))
        (setq *BA
          (- (dxf 51 entl) (dxf 50 entl)))
        (if (< *BA 0)
          (setq *BA (+ *BA (* pi 2))))
        (adddev (* *BA (+ *IBR (* *KF *TH))))
        (setq *BA (/ (* *BA 180) pi)))
      ( (princ
          (strcat
            "  <Invalid>"
            "\nYou must select "
            "an ARC or LINE entity.")))))
   (princ))
;End-of-file


;ALL DIMENSIONS

(vl-load-com)

(defun DP (side / *error* doc svnames svvals styht dpss n pl cw inc pt1 pt3 pt2 ang1 ang2 dtxt pt4)

  (defun *error* (errmsg)
    (if (not (wcmatch errmsg "Function cancelled,quit / exit abort,console break"))
      (princ (strcat "\nError: " errmsg))
    ); if
    (mapcar 'setvar svnames svvals); reset System Variables
    (vla-endundomark doc)
    (princ)
  ); defun -- *error*

  (vla-startundomark (setq doc (vla-get-activedocument (vlax-get-acad-object))))
  (setq ; System Variable saving/resetting without separate variables for each:
    svnames '(osmode cmdecho blipmode clayer)
    svvals (mapcar 'getvar svnames)
  ); setq
  (mapcar 'setvar svnames '(0 0)); turn off Osnap, command echoing
  (command "_.layer" "_make" "A-ANNO-DIMS" "_color" 2 "" "");; <---EDIT Layer name/color as desired
  (setq styht (cdr (assoc 40 (tblsearch "style" (getvar 'dimtxsty))))); height of text style in current dimension style
  (if (= styht 0.0) (setq styht (* (getvar 'dimtxt) (getvar 'dimscale)))); if above is non-fixed-height

  (prompt (strcat "\nTo Dimension Polyline(s) on the " side "side,"))
  (if (setq dpss (ssget "_:L" '((0 . "*POLYLINE"))))
    (progn ; then [can omit this if you never use blips; also two other lines below]
      (setvar 'blipmode 0); [omit if you never use blips and omitted line above]
      (repeat (setq n (sslength dpss)); step through selection set
        (setq pl (ssname dpss (setq n (1- n))))
        (if (= (logand (cdr (assoc 70 (entget pl))) 88) 0)
          ;; not 3D or mesh [88 = 8 (3D) + 16 (polygon mesh) + 64 (polyface mesh)]
          (progn ; then
            (setq pl (vlax-ename->vla-object pl))
            (vla-offset pl styht); temporary
            (setq cw (< (vla-get-area (vlax-ename->vla-object (entlast))) (vla-get-area pl)))
              ;; clockwise for closed or clearly inside/outside open; may not give
              ;; desired result for open without obvious inside/outside
            (entdel (entlast))
            (repeat (setq inc (fix (vlax-curve-getEndParam pl))); segments
              (setq
                pt1 (vlax-curve-getPointAtParam pl inc)
                pt3 (vlax-curve-getPointAtParam pl (1- inc))
              ); setq
              (if (not (equal pt1 pt3 1e-8)); not coincident vertices
                (progn ; then -- proceed to dimension segment
                  (setq
                    pt2 (vlax-curve-getPointAtParam pl (- inc 0.5)); segment midpoint
                    ang1 (angle pt1 pt2)
                    ang2 (angle pt2 pt3)
                  ); setq
                  (if
                    (or ; line segment?
                      (equal ang1 ang2 1e-8); any non-0 direction or both reading as 0 or 2 pi
                      (equal (abs (- ang2 ang1)) (* pi 2) 1e-8); 0-degree with one reading as 2 pi +/-
                    ); or
                    (command "_.dimaligned" pt1 pt3); then [leaves at dimension line location prompt]
                    (progn ; else [arc segment]
                      (setq dtxt
                        (rtos
                          (abs ; length along arc segment
                            (- (vlax-curve-getDistAtParam pl inc) (vlax-curve-getDistAtParam pl (1- inc)))
                          ); abs
                          (getvar 'DIMLUNITS) 2 ;; [include mode/precision here if current dimension style's settings not desired]
                        ); rtos
                      ); setq
                      (if (wcmatch dtxt "*/*"); includes fraction?
                        ;; can omit this entire (if) function if you don't use stacked fractions
                        (setq dtxt ; stack it
                          (strcat
                            "\\A1;"
                            (vl-string-subst ";}\"" "\""
;                              (vl-string-subst "#" "/"
                                ;; remove ; from beginning of above line and its closing parenthesis line
                                ;; below to make diagonal stack [makes horizontal-line stack without]
                                (vl-string-subst "{\\H0.875x;\\S" " " dtxt)
                                  ;; change 0.875 ratio to agree with Dimension Style's setting
;                              ); -subst
                            ); -subst
                          ); strcat & dtxt
                        ); setq
                      ); if
                      (command
                        "_.dimangular" ""
                        (inters ; arc center
                          (setq pt4 (mapcar '/ (mapcar '+ pt1 pt2) '(2 2 2)))
                          (polar pt4 (+ (angle pt1 pt2) (/ pi 2)) 1)
                          (setq pt4 (mapcar '/ (mapcar '+ pt2 pt3) '(2 2 2)))
                          (polar pt4 (+ (angle pt2 pt3) (/ pi 2)) 1)
                          nil
                        ); inters
                        pt1 pt3
                        "_text" dtxt
                      ); command [leaves at dimension line location prompt]
                    ); progn
                  ); if
                  (command ; complete Dimension: dimension line location
                    (polar
                      pt2
                      (apply ; angle
                        (if (or (and cw (= side "in")) (and (not cw) (= side "out"))) '- '+)
                        (list
                          (angle '(0 0 0) (vlax-curve-getFirstDeriv pl (- inc 0.5)))
                          (/ pi 2)
                        ); list
                      ); apply
                      (* styht 1.5); distance
                        ;; [If you don't use stacked fractions, consider using styht without multiplier]
                    ); polar
                  ); command
                ); progn
              ); if [not coincident]
              (setq inc (1- inc))
            ); repeat [segments]
          ); progn -- then [LW/2D]
        ); if
      ); repeat [Polylines]
    ); progn -- then [valid selection]; [omit if you never use blips and omitted 2 lines above]
    (prompt "\nNo Polyline(s) on unlocked Layer(s) selected."); else
  ); if

  (mapcar 'setvar svnames svvals); reset System Variables
  (vla-endundomark doc)
  (princ)
); defun -- DP

(defun C:DPI () (DP "in")); = Dimension Polyline Inside
(defun C:DPO () (DP "out")); = Dimension Polyline Outside

(prompt "\nType DPI to Dimension Polyline(s) on the Inside, DPO to do so on the Outside.")